#include <iostream>
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <math.h>
#include <float.h>

using namespace std;

/////////////////////////////////////////////////////////////
// Variables Globales
/////////////////////////////////////////////////////////////
// Taille de la fenêtre
unsigned width  = 600;
unsigned height = 600;

// Couleurs
double RED  [3] = {1, 0, 0};
double GREEN[3] = {0, 1, 0};
double BLUE [3] = {0, 0, 1};
double CYAN [3] = {0, 1, 1};

// Mouse X, Y
double x_0, y_0;		// Start
double x_c, y_c;		// Current

// Lumiere
GLfloat lightpos[4] = { 10.0, 10.0, -30.0, 0.0 };
GLfloat lightamb[4] = { 0.7, 0.7, 0.7, 1.0 };
GLfloat lightdif[4] = { 0.5, 0.5, 0.5, 1.0 };
GLfloat lightspe[4] = { 0.5, 0.5, 0.5, 1.0 };

// Other
int		pressed_button =-1;
int     obj_selected   = 0;

double  obj_rotat_0;
double  obj_scale_0;
double  obj_rotat[3] = {0.0, 0.0, 0.0};
double  obj_scale[3] = {1.0, 1.0, 1.0};

double  rayon[4] = {0.9, 0.7, 0.5, 0.05}; // Corps, Torse, Tete, Bouton
double  cone [2] = {0.1, 1.0};


/////////////////////////////////////////////////////////////
// Other
/////////////////////////////////////////////////////////////
double* cross_product(double* v1, double* v2) {
	double* v3 = new double[3];

	v3[0] = v1[1]*v2[2] - v1[2]*v2[1];
	v3[1] = v1[2]*v2[0] - v1[0]*v2[2];
	v3[2] = v1[0]*v2[1] - v1[1]*v2[0];

	return v3;
}

double scalar_product(double* v1, double* v2) {
	return v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2];
}

double dist(float x0, float y0, float x1, float y1) {
  return sqrt((x0-x1)*(x0-x1) + (y0-y1)*(y0-y1));
}

/////////////////////////////////////////////////////////////
// Other
/////////////////////////////////////////////////////////////
void enBlanc() {
    float ambiant[3] = {0.4, 0.4, 0.4};
    float diffuse[3] = {0.2, 0.2, 0.2};
    float specular[3] = {0.4, 0.4, 0.5};

    glMaterialfv(GL_FRONT, GL_AMBIENT, ambiant);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    glMaterialf(GL_FRONT, GL_SHININESS, 1200);
}

void enNoir() {
  float diffuse2[3] = {0.05, 0.05, 0.05};
  glMaterialfv(GL_FRONT, GL_AMBIENT, diffuse2);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse2);
  glMaterialfv(GL_FRONT, GL_SPECULAR, diffuse2);
  glMaterialf(GL_FRONT, GL_SHININESS, 100);
}

void enRouge() {
  float diffuse3[3] = {0.5, 0.05, 0.05};
  glMaterialfv(GL_FRONT, GL_AMBIENT  , diffuse3);
  glMaterialfv(GL_FRONT, GL_DIFFUSE  , diffuse3);
  glMaterialfv(GL_FRONT, GL_SPECULAR , diffuse3);
  glMaterialf (GL_FRONT, GL_SHININESS, 100);
}

/////////////////////////////////////////////////////////////
// Commandes
/////////////////////////////////////////////////////////////
void motion(int x, int y) {
	x_c = x;
    y_c = y; 

	obj_scale[obj_selected] = obj_scale_0 +       (y_c-y_0)/float(height);
	obj_rotat[obj_selected] = obj_rotat_0 + 100.0*(x_c-x_0)/float( width);

	glutPostRedisplay();
}

void mouse(int button, int state, int x, int y) {
    x_0 = x;
    y_0 = y; 

	obj_rotat_0 = obj_rotat[obj_selected];
    obj_scale_0 = obj_scale[obj_selected];
	
    glutPostRedisplay();
}

void keyboard(unsigned char key, int x, int y) {
    switch (key) {
    case 27:    // Echap
        exit(0);
	case ' ':
		obj_rotat[0] = 0.0;
		obj_rotat[1] = 0.0;
		obj_rotat[2] = 0.0;

		obj_scale[0] = 1.0;
		obj_scale[1] = 1.0;
		obj_scale[2] = 1.0;
		break;
	case 'b': 
		obj_selected = 0; 
		break;
	case 't': 
		obj_selected = 1;
		break;
	case 'h': 
		obj_selected = 2; 
		break;
    default:
        return;
    }
    glutPostRedisplay();
}

/////////////////////////////////////////////////////////////
// Fonctions Menu
/////////////////////////////////////////////////////////////
void menu(int item) {obj_selected = item;}

/////////////////////////////////////////////////////////////
// Fonctions Ecran
/////////////////////////////////////////////////////////////
void window_reshape(int w, int h) {
    width  = w;
    height = h;
    glViewport(0, 0, w, h);
    glutPostRedisplay();
}

void draw_objects() {
	// Projection 3D
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (double)width/height, 0.001, 100);

	// Positionnement Caméra
	gluLookAt( 0, 0,-5,         // Camera Position
			   0, 0, 0,			// Camera Direction
			   0, 1, 0);		// Camera Up

	// Vue Caméra
	glMatrixMode(GL_MODELVIEW);  
	glPushMatrix();
    glScalef(0.3, 0.3, 0.3);

    // On trace le sol
    glRotatef(-30, 1, 0, 0);
    glRotatef(-20, 0, 1, 0);
    glTranslatef(0.0, -1.0, 0.0);

    enBlanc();
    glBegin(GL_POLYGON);
    glVertex3f(-10, 0, -10);
    glVertex3f(-10, 0, 10);
    glVertex3f(10, 0, 10);
    glVertex3f(10, 0, -10);
    glEnd();

    // On trace le corps
	glPushMatrix();
    glTranslatef(0.0, rayon[0], 0.0);
    glutSolidSphere(rayon[0], 20, 20);
	glPopMatrix();
    
	/////////////////////////////////////////
	// A vous
	/////////////////////////////////////////
	
	glPopMatrix();
}

void draw_scene(void) {
    // Initialisation de la fenêtre
    glPushMatrix();
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
   
	// Light
	glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
	glLightfv(GL_LIGHT0, GL_AMBIENT , lightamb);
	glLightfv(GL_LIGHT0, GL_DIFFUSE , lightdif);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightspe);

	// Enable
	glEnable(GL_POINT_SMOOTH);
	glEnable(GL_POINT_SIZE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

    // Création des objets
    draw_objects();

    // Swap pour afficher la prochaine fenêtre
    glutSwapBuffers();
    glPopMatrix();
}

/////////////////////////////////////////////////////////////
// Main
/////////////////////////////////////////////////////////////
int main(int argc, char *argv[]) {
    // Initialisation de Glut
    glutInit           (&argc, argv);
    glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE | GLUT_MULTISAMPLE | GLUT_STENCIL);

    // Taille et position de la fenêtre
	glutInitWindowSize    (width, height);    // Window Size     600x600
    glutInitWindowPosition(  200,      0);

    // Création et Titre de la fenêtre
    glutCreateWindow("Synthese d'Image 1");

    // Fonction d'affichage (Renvoi à la fonction draw_scene)
    glutDisplayFunc(draw_scene);

    // Fonctions liers aux différentes commandes
    glutMouseFunc   (   mouse);
	glutMotionFunc  (  motion);
    glutKeyboardFunc(keyboard);

    // Fonction lier à la mise à l'échelle de la fenêtre
    glutReshapeFunc (window_reshape);

	glutCreateMenu(menu);
	glutAddMenuEntry("Bouge le corps", 0);
	glutAddMenuEntry("Bouge le torse", 1);
	glutAddMenuEntry("Bouge la tete" , 2);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

    glutSwapBuffers();
    glutMainLoop();
    return 0;
}
